@@ -21,6 +21,91 @@
 use diagnostics;
 use strict;
 
+
+my %knownattachments;
+
+# This routine quoteUrls contains inspirations from the HTML::FromText CPAN
+# module by Gareth Rees <garethr@cre.canon.co.uk>.  It has been heavily hacked,
+# all that is really recognizable from the original is bits of the regular
+# expressions.
+
+sub quoteUrls {
+    my $text = shift;		# Take a copy; don't modify in-place.
+    return $text unless $text;
+
+    my $base = Param('urlbase');
+
+    my $protocol = join '|',
+    qw(afs cid ftp gopher http https mid news nntp prospero telnet wais);
+
+    my %options = ( metachars => 1, @_ );
+
+    my $count = 0;
+
+    # Now, quote any "#" characters so they won't confuse stuff later
+    $text =~ s/#/%#/g;
+
+    # Next, find anything that looks like a URL or an email address and
+    # pull them out the the text, replacing them with a "##<digits>##
+    # marker, and writing them into an array.  All this confusion is
+    # necessary so that we don't match on something we've already replaced,
+    # which can happen if you do multiple s///g operations.
+
+    my @things;
+    while ($text =~ s%((mailto:)?([\w\.\-\+\=]+\@\w+(?:\.\w+)+)\b|
+                       (\b((?:$protocol):\S+[\w/])))%"##$count##"%exo) {
+        my $item = $&;
+
+        $item = value_quote($item);
+
+        if ($item !~ m/^$protocol:/o && $item !~ /^mailto:/) {
+            # We must have grabbed this one because it looks like an email
+            # address.
+            $item = qq{<A HREF="mailto:$item">$item</A>};
+        } else {
+            $item = qq{<A HREF="$item">$item</A>};
+        }
+
+        $things[$count++] = $item;
+    }
+    while ($text =~ s/\bbug(\s|%\#)*(\d+)/"##$count##"/ei) {
+        my $item = $&;
+        my $num = $2;
+        $item = value_quote($item); # Not really necessary, since we know
+                                # there's no special chars in it.
+        $item = qq{<A HREF="${base}show_bug.cgi?id=$num">$item</A>};
+        $things[$count++] = $item;
+    }
+    while ($text =~ s/\*\*\* This bug has been marked as a duplicate of (\d+) \*\*\*/"##$count##"/ei) {
+        my $item = $&;
+        my $num = $1;
+        $item =~ s@\d+@<A HREF="${base}show_bug.cgi?id=$num">$num</A>@;
+        $things[$count++] = $item;
+    }
+    while ($text =~ s/Created an attachment \(id=(\d+)\)/"##$count##"/e) {
+        my $item = $&;
+        my $num = $1;
+        if (exists $knownattachments{$num}) {
+            $item = qq{<A HREF="showattachment.cgi?attach_id=$num">$item</A>};
+        }
+        $things[$count++] = $item;
+    }
+
+    $text = value_quote($text);
+
+    # Stuff everything back from the array.
+    for (my $i=0 ; $i<$count ; $i++) {
+        $text =~ s/##$i##/$things[$i]/e;
+    }
+
+    # And undo the quoting of "#" characters.
+    $text =~ s/%#/#/g;
+
+    return $text;
+}
+
+quietly_check_login();
+
 my $query = "
 select
         bug_id,
@@ -40,9 +125,11 @@ select
 	target_milestone,
 	qa_contact,
 	status_whiteboard,
-        date_format(creation_ts,'Y-m-d')
+        date_format(creation_ts,'Y-m-d'),
+        groupset
 from bugs
-where bug_id = $::FORM{'id'}";
+where bug_id = $::FORM{'id'}
+and bugs.groupset & $::usergroupset = bugs.groupset";
 
 SendSQL($query);
 my %bug;
@@ -53,7 +140,8 @@ if (@row = FetchSQLData()) {
 		       "op_sys", "bug_status", "resolution", "priority",
 		       "bug_severity", "component", "assigned_to", "reporter",
 		       "bug_file_loc", "short_desc", "target_milestone",
-                       "qa_contact", "status_whiteboard", "creation_ts") {
+                       "qa_contact", "status_whiteboard", "creation_ts",
+                       "groupset") {
 	$bug{$field} = shift @row;
 	if (!defined $bug{$field}) {
 	    $bug{$field} = "";
@@ -110,7 +198,7 @@ print "
 <INPUT TYPE=HIDDEN NAME=\"id\" VALUE=$::FORM{'id'}>
 <INPUT TYPE=HIDDEN NAME=\"was_assigned_to\" VALUE=\"$bug{'assigned_to'}\">
   <TABLE CELLSPACING=0 CELLPADDING=0 BORDER=0><TR>
-    <TD ALIGN=RIGHT><B>Bug#:</B></TD><TD>$bug{'bug_id'}</TD>
+    <TD ALIGN=RIGHT><B>Bug#:</B></TD><TD><A HREF=\"show_bug.cgi?id=$bug{'bug_id'}\">$bug{'bug_id'}</A></TD>
     <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html#rep_platform\">Platform:</A></B></TD>
     <TD><SELECT NAME=rep_platform>$platform_popup</SELECT></TD>
     <TD ALIGN=RIGHT><B>Version:</B></TD>
@@ -206,17 +294,48 @@ if (Param("usestatuswhiteboard")) {
   </TR>";
 }
 
+print "<tr><td align=right><B>Attachments:</b></td>\n";
+SendSQL("select attach_id, creation_ts, description from attachments where bug_id = $::FORM{'id'}");
+while (MoreSQLData()) {
+    my ($id, $date, $desc) = (FetchSQLData());
+    if ($date =~ /^(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/) {
+        $date = "$3/$4/$2 $5:$6";
+    }
+    my $link = "showattachment.cgi?attach_id=$id";
+    $desc = value_quote($desc);
+    print qq{<td><a href="$link">$date</a></td><td colspan=4>$desc</td></tr><tr><td></td>};
+    $knownattachments{$id} = 1;
+}
+print "<td colspan=6><a href=createattachment.cgi?id=$::FORM{'id'}>Create a new attachment</a> (proposed patch, testcase, etc.)</td></tr>\n";
+
 
 print "
 </TABLE>
 <br>
 <B>Additional Comments:</B>
 <BR>
-<TEXTAREA WRAP=HARD NAME=comment ROWS=5 COLS=80></TEXTAREA><BR>
-<br>
+<TEXTAREA WRAP=HARD NAME=comment ROWS=5 COLS=80></TEXTAREA><BR>";
+
+
+if ($::usergroupset ne '0') {
+    SendSQL("select bit, description, (bit & $bug{'groupset'} != 0) from groups where bit & $::usergroupset != 0 and isbuggroup != 0 order by bit");
+    while (MoreSQLData()) {
+        my ($bit, $description, $ison) = (FetchSQLData());
+        my $check0 = !$ison ? " SELECTED" : "";
+        my $check1 = $ison ? " SELECTED" : "";
+        print "<select name=bit-$bit><option value=0$check0>\n";
+        print "People not in the \"$description\" group can see this bug\n";
+        print "<option value=1$check1>\n";
+        print "Only people in the \"$description\" group can see this bug\n";
+        print "</select><br>\n";
+    }
+}
+
+print "<br>
 <INPUT TYPE=radio NAME=knob VALUE=none CHECKED>
         Leave as <b>$bug{'bug_status'} $bug{'resolution'}</b><br>";
 
+
 # knum is which knob number we're generating, in javascript terms.
 
 my $knum = 1;
@@ -284,7 +403,9 @@ print "
 <td align=right>Opened:&nbsp;$bug{'creation_ts'}</td></tr></table>
 <HR>
 <PRE>
-" . html_quote($bug{'long_desc'}) . "
+";
+print quoteUrls($bug{'long_desc'}, email=>1, urls=>1);
+print "
 </PRE>
 <HR>\n";
 
